package java.io;

import java.security.AccessController;
import sun.security.action.GetPropertyAction;

public class BufferedWriter extends Writer
{
    private Writer out;
    private char[] cb;
    private int nChars;
    private int nextChar;
    private static int defaultCharBufferSize;
    private String lineSeparator;
    
    public BufferedWriter(Writer writer) {
        this(writer, BufferedWriter.defaultCharBufferSize);
    }
    
    public BufferedWriter(Writer out, int nChars) {
        super(out);
        if (nChars <= 0) {
            throw new IllegalArgumentException("Buffer size <= 0");
        }
        this.out = out;
        this.cb = new char[nChars];
        this.nChars = nChars;
        this.nextChar = 0;
        this.lineSeparator = (String)AccessController.doPrivileged(new GetPropertyAction("line.separator"));
        return;
    }
    
    private void ensureOpen() throws IOException {
        if (this.out == null) {
            throw new IOException("Stream closed");
        }
        return;
    }
    
    void flushBuffer() throws IOException {
        synchronized (this.lock) {
            this.ensureOpen();
            if (this.nextChar == 0) {
                return;
            }
            this.out.write(this.cb, 0, this.nextChar);
            this.nextChar = 0;
        }
    }
    
    @Override
    public void write(int n) throws IOException {
        synchronized (this.lock) {
            this.ensureOpen();
            if (this.nextChar >= this.nChars) {
                this.flushBuffer();
            }
            this.cb[this.nextChar++] = (char)n;
        }
    }
    
    private int min(int n, int n2) {
        if (n < n2) {
            return n;
        }
        return n2;
    }
    
    @Override
    public void write(char[] array, int i, int n) throws IOException {
        synchronized (this.lock) {
            this.ensureOpen();
            if (i < 0 || i > array.length || n < 0 || i + n > array.length || i + n < 0) {
                throw new IndexOutOfBoundsException();
            }
            if (n == 0) {
                return;
            }
            if (n >= this.nChars) {
                this.flushBuffer();
                this.out.write(array, i, n);
                return;
            }
            int n2 = i + n;
            while (i < n2) {
                int min = this.min(this.nChars - this.nextChar, n2 - i);
                System.arraycopy(array, i, this.cb, this.nextChar, min);
                i += min;
                this.nextChar += min;
                if (this.nextChar >= this.nChars) {
                    this.flushBuffer();
                }
                else {
                    continue;
                }
            }
        }
    }
    
    @Override
    public void write(String s, int i, int n) throws IOException {
        synchronized (this.lock) {
            this.ensureOpen();
            int n2 = i + n;
            while (i < n2) {
                int min = this.min(this.nChars - this.nextChar, n2 - i);
                s.getChars(i, i + min, this.cb, this.nextChar);
                i += min;
                this.nextChar += min;
                if (this.nextChar >= this.nChars) {
                    this.flushBuffer();
                }
                else {
                    continue;
                }
            }
        }
    }
    
    public void newLine() throws IOException {
        super.write(this.lineSeparator);
    }
    
    @Override
    public void flush() throws IOException {
        synchronized (this.lock) {
            this.flushBuffer();
            this.out.flush();
        }
    }
    
    @Override
    public void close() throws IOException {
        synchronized (this.lock) {
            if (this.out == null) {
                return;
            }
            try (final Writer out = this.out) {
                this.flushBuffer();
            }
            finally {
                this.out = null;
                this.cb = null;
            }
        }
    }
    
    static {
        BufferedWriter.defaultCharBufferSize = 8192;
    }
}
